# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

from customtest import *
from tablatex import tabvar


class TestTabvar(CustomTest):

    def assert_tableau(self, func, chaine, code_latex):
        code = func(chaine)
        if code != code_latex:
            print "-------"
            print "ERREUR:"
            print "Actually result is:"
            print code
            print "While expected output was:"
            print code_latex
            for i, car in enumerate(code):
                if i >= len(code_latex):
                    print 'Output too long:'
                    print code[i:]
                    break
                elif code_latex[i] != car:
                    print 'Difference:'
                    print 'char number:', i
                    print 'result:', repr(code[i:i+10])
                    print 'expected:', repr(code_latex[i:i+10])
                    break
            print "-------"
        self.assert_(code == code_latex)

    def assert_tabvar(self, chaine, code_latex):
        self.assert_tableau(tabvar.tabvar, chaine, code_latex)

    def assert_tabsign(self, chaine, code_latex):
        self.assert_tableau(tabvar.tabsign, chaine, code_latex)

    def assert_tabval(self, chaine, code_latex):
        self.assert_tableau(tabvar.tabval, chaine, code_latex)

    def test_tableau_variations(self):
        s = "x;f(x);f'(x):0;2;|>>1;0;1<<2;3;0"
        tab = \
r"""\[\begin{tabvar}{|C|CCCCCCC|}
\hline
x                                    & &0&             &        &1&      &2\\
\hline
f'(x)                                &&\dbarre&        &-       &1&+     &0\\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f(x)$}&\niveau{2}{2} &2&&\decroit&0&\croit&3\\
\hline
\end{tabvar}\]
% x;f(x);f'(x):0;2;|>>1;0;1<<2;3;0
"""
        self.assert_tabvar(s, tab)

        s = "x;f(x);f'(x):(0;2;|) >> (1;0;1) << (2;3;0)"
        tab = \
r"""\[\begin{tabvar}{|C|CCCCCCC|}
\hline
x                                    & &0&             &        &1&      &2\\
\hline
f'(x)                                &&\dbarre&        &-       &1&+     &0\\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f(x)$}&\niveau{2}{2} &2&&\decroit&0&\croit&3\\
\hline
\end{tabvar}\]
% x;f(x);f'(x):(0;2;|) >> (1;0;1) << (2;3;0)
"""
        self.assert_tabvar(s, tab)

        s = "x;f(x):-oo;+oo>>0;-oo|+oo>>+oo;-oo"
        tab = \
r"""\[\begin{tabvar}{|C|CCCCCCC|}
\hline
x                                    &-\infty             &        & &0&                                &        &+\infty\\
\hline
f'(x)                                &                    &-       &&\dbarre&                           &-       & \\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f(x)$}&\niveau{2}{2}+\infty&\decroit&-\infty&\dbarre&\niveau{2}{2}+\infty&\decroit&-\infty\\
\hline
\end{tabvar}\]
% x;f(x):-oo;+oo>>0;-oo|+oo>>+oo;-oo
"""
        self.assert_tabvar(s, tab)

        s = "x;f(x): (-oo;+oo)>>(0;-oo|+oo)>>(+oo;-oo)"
        tab = \
r"""\[\begin{tabvar}{|C|CCCCCCC|}
\hline
x                                    &-\infty             &        & &0&                                &        &+\infty\\
\hline
f'(x)                                &                    &-       &&\dbarre&                           &-       & \\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f(x)$}&\niveau{2}{2}+\infty&\decroit&-\infty&\dbarre&\niveau{2}{2}+\infty&\decroit&-\infty\\
\hline
\end{tabvar}\]
% x;f(x): (-oo;+oo)>>(0;-oo|+oo)>>(+oo;-oo)
"""
        self.assert_tabvar(s, tab)

        s = 'f(x)=(x+1)/(3x-2)'
        tab = \
r"""\[\begin{tabvar}{|C|CCCCCCC|}
\hline
x                                    &-\infty                 &        & &\frac{2}{3}&                      &        &+\infty\\
\hline
f'(x)                                &                        &-       &&\dbarre&                           &-       & \\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f(x)$}&\niveau{2}{2}\frac{1}{3}&\decroit&-\infty&\dbarre&\niveau{2}{2}+\infty&\decroit&\frac{1}{3}\\
\hline
\end{tabvar}\]
% x;f(x):(-oo;1/3) >> (2/3;-oo|+oo;|) >> (+oo;1/3)
% f(x)=(x+1)/(3x-2)
"""
        self.assert_tabvar(s, tab)

        s = "(x+1)(x+2)"
        tab = \
r"""\[\begin{tabvar}{|C|CCCCC|}
\hline
x                                 &-\infty             &        &-\frac{3}{2}&      &+\infty\\
\hline
f'(x)                             &                    &-       &0           &+     & \\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f$}&\niveau{2}{2}+\infty&\decroit&-\frac{1}{4}&\croit&+\infty\\
\hline
\end{tabvar}\]
% x;f:(-oo;+oo) >> (-3/2;-1/4) << (+oo;+oo)
% (x+1)(x+2)
"""
        self.assert_tabvar(s, tab)

        s = "x;f(x);f'(x):(0;2;|) >> (1;-oo) || (2;+oo) << (+oo;3)"
        tab = \
r"""\[\begin{tabvar}{|C|CCCCCNCCC|}
\hline
x                                    & &0&             &        &1      &\hspace*{15mm}&2      &      &+\infty\\
\hline
f'(x)                                &&\dbarre&        &-       &0      &              &0      &+     & \\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f(x)$}&\niveau{2}{2} &2&&\decroit&-\infty&              &+\infty&\croit&3\\
\hline
\end{tabvar}\]
% x;f(x);f'(x):(0;2;|) >> (1;-oo) || (2;+oo) << (+oo;3)
"""
        self.assert_tabvar(s, tab)

        s = "x^2 sur [1;+oo["
        tab = \
r"""\[\begin{tabvar}{|C|CCC|}
\hline
x                                 &1             &      &+\infty\\
\hline
f'(x)                             &              &+     & \\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f$}&\niveau{1}{2}1&\croit&+\infty\\
\hline
\end{tabvar}\]
% x;f:(1;1) << (+oo;+oo)
% x^2 sur [1;+oo[
"""
        self.assert_tabvar(s, tab)

        s = "f(x)=5*ln(x)/x+3"
        tab = \
r"""\[\begin{tabvar}{|C|CCCCCCC|}
\hline
x                                    & &0&                                      &      &\e            &        &+\infty\\
\hline
f'(x)                                &&\dbarre&                                 &+     &0             &-       & \\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f(x)$}&\niveau{1}{2}&\dbarre&\niveau{1}{2}-\infty&\croit&3 + 5 \exp{-1}&\decroit&3\\
\hline
\end{tabvar}\]
% x;f(x):(0;|-oo;|) << (e;3 + 5*exp(-1)) >> (+oo;3)
% f(x)=5*ln(x)/x+3
"""
        self.assert_tabvar(s, tab)

        s = "f(x)=3e^{2x}-12e^{x}+5"
        tab = \
r"""\[\begin{tabvar}{|C|CCCCC|}
\hline
x                                    &-\infty       &        &\ln{2}&      &+\infty\\
\hline
f'(x)                                &              &-       &0     &+     & \\
\hline
\niveau{1}{2}\raisebox{0.5em}{$f(x)$}&\niveau{2}{2}5&\decroit&-7    &\croit&+\infty\\
\hline
\end{tabvar}\]
% x;f(x):(-oo;5) >> (ln(2);-7) << (+oo;+oo)
% f(x)=3e^{2x}-12e^{x}+5
"""
        self.assert_tabvar(s, tab)

    def test_tableau_signes(self):
        s = "x: -oo;+oo// 2x+1: -- -1/2 ++// 3-x: ++ 3 --// f(x)"
        tab = \
r"""\begin{center}
\begin{tabular}{|c|ccccccc|}
\hline
$x$    & $-\infty$ &     & $-\frac{1}{2}$ &   & $3$ &     & $+\infty$ \\
\hline
$2x+1$ &           & $-$ &       0        & + &     &  +  &           \\
\hline
$3-x$  &           &  +  &                & + &  0  & $-$ &           \\
\hline
$f(x)$ &           & $-$ &       0        & + &  0  & $-$ &           \\
\hline
\end{tabular}
\end{center}
% x: -oo;+oo// 2x+1: -- -1/2 ++// 3-x: ++ 3 --// f(x)
"""
        self.assert_tabsign(s, tab)

        s = 'g(x)=(x-7/2)(x+7/2)'
        tab = \
r'''\begin{center}
\begin{tabular}{|c|ccccccc|}
\hline
$x$             & $-\infty$ &     & $-\frac{7}{2}$ &     & $\frac{7}{2}$ &   & $+\infty$ \\
\hline
$x-\frac{7}{2}$ &           & $-$ &                & $-$ &       0       & + &           \\
\hline
$x+\frac{7}{2}$ &           & $-$ &       0        &  +  &               & + &           \\
\hline
$g(x)$          &           &  +  &       0        & $-$ &       0       & + &           \\
\hline
\end{tabular}
\end{center}
% x:-oo;+oo// x-7/2: -- 7/2 ++ // x+7/2: -- -7/2 ++ // g(x)
% g(x)=(x-7/2)(x+7/2)
'''
        self.assert_tabsign(s, tab)

        s= 'f(x)=x^3-30x^2+112'
        tab = \
r"""\begin{center}
\begin{tabular}{|c|ccccccccc|}
\hline
$x$                  & $-\infty$ &     & $14 - 6 \sqrt{7}$ &   & $2$ &     & $14 + 6 \sqrt{7}$ &   & $+\infty$ \\
\hline
$f(x)$               &           & $-$ &         0         & + &  0  & $-$ &         0         & + &           \\
\hline
\end{tabular}
\end{center}
% x:-oo;+oo// x^3-30 x^2+112: -- 14 - 6*sqrt(7) ++ 2 -- 14 + 6*sqrt(7) ++ // f(x)
% f(x)=x^3-30x^2+112
"""
        self.assert_tabsign(s, tab)

        s = '(3x-2)/((x-1)^2)'
        tab = \
r'''\begin{center}
\begin{tabular}{|c|ccccccc|}
\hline
$x$                      & $-\infty$ &     & $\frac{2}{3}$ &   & $1$ &   & $+\infty$ \\
\hline
$3 x-2$                  &           & $-$ &       0       & + &     & + &           \\
\hline
$(x-1)^{2}$              &           &  +  &               & + &  0  & + &           \\
\hline
$\frac{3x-2}{(x-1)^{2}}$ &           & $-$ &       0       & + &  || & + &           \\
\hline
\end{tabular}
\end{center}
% x:-oo;+oo// 3 x-2: -- 2/3 ++ // !(x-1)^2: ++ 1 ++ // (3x-2)/((x-1)^2)
% (3x-2)/((x-1)^2)
'''
        self.assert_tabsign(s, tab)

        s = '(3x-2)/(x-1)^2'
        tab = \
r'''\begin{center}
\begin{tabular}{|c|ccccccc|}
\hline
$x$                      & $-\infty$ &     & $\frac{2}{3}$ &   & $1$ &   & $+\infty$ \\
\hline
$3 x-2$                  &           & $-$ &       0       & + &     & + &           \\
\hline
$(x-1)^{2}$              &           &  +  &               & + &  0  & + &           \\
\hline
$\frac{3x-2}{(x-1)^{2}}$ &           & $-$ &       0       & + &  || & + &           \\
\hline
\end{tabular}
\end{center}
% x:-oo;+oo// 3 x-2: -- 2/3 ++ // !(x-1)^2: ++ 1 ++ // (3x-2)/(x-1)^2
% (3x-2)/(x-1)^2
'''
        self.assert_tabsign(s, tab)

        s = '\dfrac{3x-2}{(x-1)^2}'
        tab = \
r'''\begin{center}
\begin{tabular}{|c|ccccccc|}
\hline
$x$                       & $-\infty$ &     & $\frac{2}{3}$ &   & $1$ &   & $+\infty$ \\
\hline
$3 x-2$                   &           & $-$ &       0       & + &     & + &           \\
\hline
$(x-1)^{2}$               &           &  +  &               & + &  0  & + &           \\
\hline
$\dfrac{3x-2}{(x-1)^{2}}$ &           & $-$ &       0       & + &  || & + &           \\
\hline
\end{tabular}
\end{center}
% x:-oo;+oo// 3 x-2: -- 2/3 ++ // !(x-1)^2: ++ 1 ++ // \dfrac{3x-2}{(x-1)^2}
% \dfrac{3x-2}{(x-1)^2}
'''
        self.assert_tabsign(s, tab)

        s = '- 6 x^{2} - 12 x + 4'
        tab = \
r'''\begin{center}
\begin{tabular}{|c|ccccccc|}
\hline
$x$                    & $-\infty$ &     & $-1 - \frac{\sqrt{15}}{3}$ &   & $-1 + \frac{\sqrt{15}}{3}$ &     & $+\infty$ \\
\hline
$- 6 x^{2} - 12 x + 4$ &           & $-$ &             0              & + &             0              & $-$ &           \\
\hline
\end{tabular}
\end{center}
% x:-oo;+oo// -6 x^(2)-12 x+4: -- -1 - sqrt(15)/3 ++ -1 + sqrt(15)/3 -- // - 6 x^{2} - 12 x + 4
% - 6 x^{2} - 12 x + 4
'''
        self.assert_tabsign(s, tab)

        s= "x^2 sur [1;+oo["
        tab = \
r'''\begin{center}
\begin{tabular}{|c|ccc|}
\hline
$x$     & $1$ &   & $+\infty$ \\
\hline
$x^{2}$ &     & + &           \\
\hline
\end{tabular}
\end{center}
% x:1;+oo// x^2: ++ // x^2
% x^2 sur [1;+\infty[
'''
        self.assert_tabsign(s, tab)

        s= "g(x)=\dfrac{-x+1}{\e^{x}}"
        tab = \
r'''\begin{center}
\begin{tabular}{|c|ccccc|}
\hline
$x$      & $-\infty$ &   & $1$ &     & $+\infty$ \\
\hline
$-x+1$   &           & + &  0  & $-$ &           \\
\hline
$\e^{x}$ &     0     & + &     &  +  &           \\
\hline
$g(x)$   &     ||    & + &  0  & $-$ &           \\
\hline
\end{tabular}
\end{center}
% x:-oo;+oo// -x+1: ++ 1 -- // !e^(x):-oo ++ // g(x)
% g(x)=\dfrac{-x+1}{\e^{x}}
'''
        self.assert_tabsign(s, tab)

        s= "f'(x)=1-\e^{-x+2}"
        tab = \
r'''\begin{center}
\begin{tabular}{|c|ccccc|}
\hline
$x$           & $-\infty$ &     & $2$ &   & $+\infty$ \\
\hline
$f'(x)$       &           & $-$ &  0  & + &           \\
\hline
\end{tabular}
\end{center}
% x:-oo;+oo// 1-e^(-x+2): -- 2 ++ // f'(x)
% f'(x)=1-\e^{-x+2}
'''
        self.assert_tabsign(s, tab)



    def test_tableau_valeurs(self):
        s = "f(x)=exp(x+1): 0.25: -5, -4..0 ; 0.5 ; 1, 2..6 ; 7, 10..21"
        tab = \
r"""\begin{center}
\begin{tabular}{|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|}
\hline
$x$    & $-5$ & $-4$ & $-3$   & $-2$   & $-1$ & $0$    & $0,5$ & $1$   & $2$  & $3$    & $4$     & $5$     & $6$       & $7$    & $10$       & $13$         & $16$          & $19$          & $21$ \\
\hline
$f(x)$ & $0$  & $0$  & $0,25$ & $0,25$ & $1$  & $2,75$ & $4,5$ & $7,5$ & $20$ & $54,5$ & $148,5$ & $403,5$ & $1096,75$ & $2981$ & $59874,25$ & $1202604,25$ & $24154952,75$ & $485165195,5$ & $3584912846,25$ \\
\hline
\end{tabular}
\end{center}
% f(x)=exp(x+1): 0.25: -5, -4..0 ; 0.5 ; 1, 2..6 ; 7, 10..21
"""
        self.assert_tabval(s, tab)

        s = "f(x)=exp(x+1): [0.25]: -5,-4..0 ; 0.5 ; 1,2..6 ; 7,10..21"
        tab = \
r"""\begin{center}
\begin{tabular}{|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|c|}
\hline
$x$    & $-5$ & $-4$ & $-3$   & $-2$   & $-1$ & $0$    & $0,5$ & $1$   & $2$  & $3$    & $4$     & $5$     & $6$       & $7$    & $10$       & $13$         & $16$          & $19$          & $21$ \\
\hline
$f(x)$ & $0$  & $0$  & $0,25$ & $0,25$ & $1$  & $2,75$ & $4,5$ & $7,5$ & $20$ & $54,5$ & $148,5$ & $403,5$ & $1096,75$ & $2981$ & $59874,25$ & $1202604,25$ & $24154952,75$ & $485165195,5$ & $3584912846,25$ \\
\hline
\end{tabular}
\end{center}
% f(x)=exp(x+1): [0.25]: -5,-4..0 ; 0.5 ; 1,2..6 ; 7,10..21
"""
        self.assert_tabval(s, tab)

        s = "(2x+3)^2:-10..0// 1..10"
        tab = \
r"""\begin{center}
\begin{tabular}{|c|c|c|c|c|c|c|c|c|c|c|c|}
\hline
$x$        & $-10$ & $-9$  & $-8$  & $-7$  & $-6$ & $-5$ & $-4$ & $-3$ & $-2$ & $-1$ & $0$ \\
\hline
$(2x+3)^2$ & $289$ & $225$ & $169$ & $121$ & $81$ & $49$ & $25$ & $9$  & $1$  & $1$  & $9$ \\
\hline
\end{tabular}

\begin{tabular}{|c|c|c|c|c|c|c|c|c|c|c|}
\hline
$x$        & $1$  & $2$  & $3$  & $4$   & $5$   & $6$   & $7$   & $8$   & $9$   & $10$ \\
\hline
$(2x+3)^2$ & $25$ & $49$ & $81$ & $121$ & $169$ & $225$ & $289$ & $361$ & $441$ & $529$ \\
\hline
\end{tabular}
\end{center}
% (2x+3)^2:-10..0// 1..10
"""
        self.assert_tabval(s, tab)



if __name__ == '__main__':
    unittest.main()
