#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)
from __future__ import with_statement

#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2010  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

u"""Ce script détecte tous les fichiers de WxGéométrie. Pour chacun d'entre eux,
une valeur True ou False est affectée manuellement, suivant qu'ils doivent être distribués ou non.
Les résultats sont conservés d'une fois sur l'autre, afin qu'il n'y ait que les nouveaux
fichiers à traiter. Les fichiers disparus provoquent un avertissement.
(doc à compléter)
"""


## <CONFIG>
include_paths = ['sympy', 'images', 'exemples', 'modules/scicalc', 'geolib/tests_unitaires', 'mathlib/tests_unitaires', 'doc/images']
exclude_paths = ['ressources_annexes', 'pygraph', 'preferences', 'log', 'modules/_3d', 'modules/sudoku_', 'modules/OLD']
exclude_suffixes = ['~', '.backup', '.pyc', '.pywc', '.aux', '.log', '.e4q', '.e4s', '.e4t', '.e4p', '.bat', '(copie).py', '_old.py', '.bak', '.py.old']
exclude_prefixes = ['Copie de ']
include_is_default = True
## </CONFIG>


import os, sys

_module_path = os.path.split(os.path.realpath(sys._getframe().f_code.co_filename))[0]



include_path = [os.path.normpath(path) for path in include_paths]
exclude_path = [os.path.normpath(path) for path in exclude_paths]

# on choisit comme répertoire courant le repertoire de wxgeometrie (pour Linux)
os.chdir(_module_path)

try:
    from distrib_files import paths
except ImportError:
    print("Warning: distrib_files.py not found.")
    # filename : False | True | None
    paths = {}

os.chdir('..')

existing_paths = set()
for root, dirs, files in os.walk('.'):
    existing_paths.update(os.path.join(root, f) for f in files)

previous_paths = set(paths)

deprecated_paths = previous_paths.difference(existing_paths)

for path in sorted(existing_paths):
    if any(path.endswith(suff) for suff in exclude_suffixes):
        include = False
    elif any(os.path.basename(path).startswith(pref) for pref in exclude_prefixes):
        include = False
    elif any(path.startswith('.' + os.sep + pth + os.sep) for pth in exclude_paths):
        include = False
    elif any(path.startswith('.' + os.sep + pth + os.sep) for pth in include_paths):
        include = True
    elif path in previous_paths:
        continue
    else:
        while True:
            rep = raw_input("include '%s' ? [y(es)/n(o)/q(uit)]"%path)
            if rep in 'yYoO' or (include_is_default and not rep):
                include = True
                break
            elif rep in 'nN':
                include = False
                break
            elif rep in 'qQ':
                include = None
                break
            else:
                print('Sorry, not understood.')
    if include is None:
        break
    paths[path] = include

if deprecated_paths:
    print("Warning: deprecated entries found:")
    rep = '?'
    for path in sorted(deprecated_paths):
        while True:
            if rep not in 'aA':
                rep = raw_input("remove '%s' ? [a(ll)/y(es)/n(o)/q(uit)]"%path)
            if rep in 'aAyY':
                remove = True
                break
            elif rep in 'nN':
                remove = False
                break
            elif rep in 'qQ':
                break
            else:
                print('Sorry, not understood.')
        if rep in 'qQ':
            break
        elif remove:
            del paths[path]

with open('tools/distrib_files.py', 'w') as f:
    f.write('paths = {\n' + '\n'.join(sorted(repr(k) + ': ' + repr(v) + ',' for (k,v) in paths.items())) + '\n}')
